package java.nio.charset;

import java.util.HashMap;
import java.lang.ref.WeakReference;
import java.util.Map;
import java.nio.BufferOverflowException;
import java.nio.BufferUnderflowException;

public class CoderResult
{
    private static final int CR_UNDERFLOW = 0;
    private static final int CR_OVERFLOW = 1;
    private static final int CR_ERROR_MIN = 2;
    private static final int CR_MALFORMED = 2;
    private static final int CR_UNMAPPABLE = 3;
    private static final String[] names;
    private final int type;
    private final int length;
    public static final CoderResult UNDERFLOW;
    public static final CoderResult OVERFLOW;
    private static CoderResult.Cache malformedCache;
    private static CoderResult.Cache unmappableCache;
    
    private CoderResult(int type, int length) {
        this.type = type;
        this.length = length;
    }
    
    @Override
    public String toString() {
        String str = CoderResult.names[this.type];
        return this.isError() ? (str + "[" + this.length + "]") : str;
    }
    
    public boolean isUnderflow() {
        return this.type == 0;
    }
    
    public boolean isOverflow() {
        return this.type == 1;
    }
    
    public boolean isError() {
        return this.type >= 2;
    }
    
    public boolean isMalformed() {
        return this.type == 2;
    }
    
    public boolean isUnmappable() {
        return this.type == 3;
    }
    
    public int length() {
        if (!this.isError()) {
            throw new UnsupportedOperationException();
        }
        return this.length;
    }
    
    public static CoderResult malformedForLength(int n) {
        return CoderResult.Cache.access$200(CoderResult.malformedCache, n);
    }
    
    public static CoderResult unmappableForLength(int n) {
        return CoderResult.Cache.access$200(CoderResult.unmappableCache, n);
    }
    
    public void throwException() throws CharacterCodingException {
        switch (this.type) {
            case 0: {
                throw new BufferUnderflowException();
            }
            case 1: {
                throw new BufferOverflowException();
            }
            case 2: {
                throw new MalformedInputException(this.length);
            }
            case 3: {
                throw new UnmappableCharacterException(this.length);
            }
            default: {
                assert false;
                return;
            }
        }
    }
    
    CoderResult(int n, int n2, CoderResult.1 cache) {
        this(n, n2);
    }
    
    static {
        names = new String[] { "UNDERFLOW", "OVERFLOW", "MALFORMED", "UNMAPPABLE" };
        UNDERFLOW = new CoderResult(0, 0);
        OVERFLOW = new CoderResult(1, 0);
        CoderResult.malformedCache = new CoderResult.Cache() {
            public CoderResult create(int n) {
                return new CoderResult(2, n);
            }
        };
        CoderResult.unmappableCache = new CoderResult.Cache() {
            public CoderResult create(int n) {
                return new CoderResult(3, n);
            }
        };
    }
    
    private abstract static class Cache
    {
        private Map<Integer, WeakReference<CoderResult>> cache;
        
        private Cache() {
            this.cache = null;
        }
        
        protected abstract CoderResult create(int p0);
        
        private synchronized CoderResult get(int value) {
            if (value <= 0) {
                throw new IllegalArgumentException("Non-positive length");
            }
            Integer n = new Integer(value);
            CoderResult create = null;
            if (this.cache == null) {
                this.cache = new HashMap();
            }
            else {
                WeakReference<Object> weakReference;
                if ((weakReference = (WeakReference)this.cache.get(n)) != null) {
                    create = (CoderResult)weakReference.get();
                }
            }
            if (create == null) {
                create = this.create(value);
                this.cache.put(n, new WeakReference(create));
                return create;
            }
            return create;
        }
        
        static CoderResult access$200(CoderResult.Cache cache, int n) {
            return cache.get(n);
        }
    }
}
